import React, { useEffect, useState } from 'react';
import { connect } from 'react-redux';
import { setButton } from 'app/shared/layout/floating/floating.reducer';
import { IRootState } from 'app/shared/reducers';
import { getChartQuery, getEntity, reset } from './chart.reducer';
import { RouteComponentProps } from 'react-router-dom';
import JsxParser from 'react-jsx-parser';
import { Box, Typography, Tooltip } from '@material-ui/core';
import AlertBox from '@material-ui/lab/Alert';
import { Translate, translate } from 'react-jhipster';
import { isEmpty } from 'lodash';
import getClasses from 'app/modules/dashboard/style';
import { ChartViewMap } from 'app/modules/dashboard/charts/chart-view-map';
import {
  createChartFromEntity,
  isTypeMap,
  graphWithLabel,
  defaultChartValues,
  calculatePercentage,
  parserComponents,
  saveChartConfig,
  getConfByChartId,
  isHintChanged
} from 'app/shared/util/chart-utils';
import { DEFAULT_CHART_SIZE } from 'app/config/constants';

export interface IChartVisualizationProps extends StateProps, DispatchProps, RouteComponentProps<{ id: string }> {}

export const ChartVisualization = (props: IChartVisualizationProps) => {
  const { entity, chartDataset } = props;
  const [chart, setChart] = useState(null);
  const classes = getClasses();
  const [chartValues, setChartValues] = useState(defaultChartValues);

  useEffect(() => {
    props.setButton([
      {
        label: translate('global.entity.action.back'),
        icon: 'arrow_back',
        action: '/chart',
        isLink: true
      }
    ]);
    props.getEntity(props.match.params.id);
    return () => props.reset();
  }, []);

  useEffect(() => {
    if (entity && entity.id) {
      props.getChartQuery(entity.id);
    }
  }, [entity]);

  useEffect(() => {
    if (chartDataset && !chart) {
      if (!isTypeMap(entity.chartView)) {
        if (graphWithLabel(entity.chartView)) {
          const id = entity.id;
          saveChartConfig(id, chartDataset);
        }
        setChart(createChartFromEntity(entity.chartView, chartDataset));
      }
    }
  }, [chartDataset]);

  return (
    <div className={classes.divChart}>
      {entity && (
        <Tooltip title={entity.description || ''} disableHoverListener={isEmpty(entity.description)} placement="bottom" arrow>
          <Typography paragraph component="h3" variant="h5" align="center">
            {entity.title}
          </Typography>
        </Tooltip>
      )}
      {entity &&
        chartDataset && [
          chartDataset.length === 0 ? (
            <AlertBox key="item" severity="warning">
              <Translate contentKey="entity.noResultFound">Nessun risultato trovato</Translate>
            </AlertBox>
          ) : (
            [
              isTypeMap(entity.chartView) ? (
                <ChartViewMap
                  height={DEFAULT_CHART_SIZE}
                  key="item"
                  disableDragEvent={() => void 0}
                  chartView={entity.chartView}
                  chartDataset={chartDataset}
                />
              ) : (
                <JsxParser
                  key="item"
                  bindings={{
                    id: entity.id,
                    chartDataset,
                    colorAndItems: getConfByChartId(entity.id),
                    disableDragEvent: () => void 0,
                    enableDragEvent: () => void 0,
                    eventOver(v) {
                      isHintChanged(chartValues.hint, v) &&
                        setChartValues({ id: entity.id, hint: v, percentage: calculatePercentage(chartDataset, v) });
                    },
                    eventOut(v) {
                      setChartValues(defaultChartValues);
                    },
                    chartValues
                  }}
                  components={{ ...parserComponents }}
                  jsx={chart}
                />
              )
            ]
          )
        ]}
    </div>
  );
};

const mapStateToProps = (storeState: IRootState) => ({
  chartDataset: storeState.chart.chartDataset,
  entity: storeState.chart.entity
});

const mapDispatchToProps = {
  getChartQuery,
  getEntity,
  setButton,
  reset
};

type StateProps = ReturnType<typeof mapStateToProps>;
type DispatchProps = typeof mapDispatchToProps;

export default connect(mapStateToProps, mapDispatchToProps)(ChartVisualization);
