import React, { useState, useEffect } from 'react';
import { connect } from 'react-redux';
import { Link, RouteComponentProps } from 'react-router-dom';
import { Translate } from 'react-jhipster';
import { getSortState } from 'app/shared/layout/pagination/paginator-utils';
import { IRootState } from 'app/shared/reducers';
import { getEntities, deleteEntity, updateEntity, removeUserFromChart } from './chart.reducer';
import { ITEMS_PER_PAGE } from 'app/shared/util/pagination.constants';
import { has, isUndefined } from 'lodash';
import { MenuItem } from '@material-ui/core';
import Table from '@material-ui/core/Table';
import TableBody from '@material-ui/core/TableBody';
import TableCell from '@material-ui/core/TableCell';
import TableContainer from '@material-ui/core/TableContainer';
import TableHead from '@material-ui/core/TableHead';
import TableRow from '@material-ui/core/TableRow';
import Paper from '@material-ui/core/Paper';
import DropdownMenu from 'app/shared/layout/menus/dropdown-menu';
import Box from '@material-ui/core/Box';
import MoreVertIcon from '@material-ui/icons/MoreVert';
import { setButton } from 'app/shared/layout/floating/floating.reducer';
import { defaultValue } from 'app/shared/model/chart.model';
import DialogBox from 'app/shared/components/dialog';
import AlertBox from '@material-ui/lab/Alert';
import { Paginator } from 'app/shared/layout/pagination/paginator';
import { resetSteps } from 'app/shared/layout/header/breadcrumb.reducer';
import { translate } from 'app/shared/util/common-utils';
import { hasAnyAuthority } from 'app/shared/auth/private-route';
import { AUTHORITIES } from 'app/config/constants';
import { isTypeMap } from 'app/shared/util/chart-utils';

export interface IChartProps extends StateProps, DispatchProps, RouteComponentProps<{ url: string }> {}

export const Chart = (props: IChartProps) => {
  const { currentUser } = props;
  const [isDeleteDialogOpen, setIsDeleteDialogOpen] = useState(false);
  const [entityToBeDeleted, setEntityToBeDeleted] = useState(defaultValue);
  const [pagination, setPagination] = useState(getSortState(props.location, ITEMS_PER_PAGE, 'title'));
  const isRootOrAdmin = hasAnyAuthority(props.authorities, [AUTHORITIES.ROOT, AUTHORITIES.ADMIN]);

  const getAllEntities = () => {
    props.getEntities();
  };

  const sortEntities = () => {
    getAllEntities();
    props.history.push(
      `${props.location.pathname}?page=${pagination.activePage}&size=${pagination.itemsPerPage}&sort=${pagination.sort},${pagination.order}`
    );
  };

  useEffect(() => {
    sortEntities();
  }, [pagination.activePage, pagination.order, pagination.sort]);

  useEffect(() => {
    props.resetSteps();
    if (isRootOrAdmin) {
      props.setButton([
        {
          label: translate('efraudyApp.rule.home.createLabel'),
          icon: 'add',
          action: 'chart/new',
          isLink: true
        }
      ]);
    } else {
      props.setButton([]);
    }
  }, []);

  const handlePagination = (e, currentPage) =>
    setPagination({
      ...pagination,
      activePage: currentPage
    });

  const openDeleteDialog = alert => e => {
    setEntityToBeDeleted(alert);
    setIsDeleteDialogOpen(true);
  };

  const closeDeleteDialog = () => {
    setEntityToBeDeleted(defaultValue);
    setIsDeleteDialogOpen(false);
  };

  const deleteChart = e => {
    if (!has(entityToBeDeleted, 'id') || isUndefined(entityToBeDeleted.id)) {
      throw "Can't delete an entity without an ID";
    }
    props.deleteEntity(entityToBeDeleted.id);
    closeDeleteDialog();
  };

  const { chartList, match, loading, totalItems } = props;
  return (
    <div>
      {chartList && chartList.length > 0 ? (
        <div>
          <TableContainer component={Paper}>
            <Table aria-label="rule table list">
              <TableHead>
                <TableRow>
                  <TableCell>
                    <Box display="flex" alignItems="center" className="pointer">
                      <Translate contentKey="efraudyApp.chart.title">Name</Translate>
                    </Box>
                  </TableCell>
                  <TableCell>
                    <Translate contentKey="efraudyApp.chart.description">Description</Translate>
                  </TableCell>
                  <TableCell>
                    <Box display="flex" alignItems="center" className="pointer">
                      <Translate contentKey="efraudyApp.chart.typeChart">Tipo di grafico</Translate>
                    </Box>
                  </TableCell>
                  <TableCell></TableCell>
                </TableRow>
              </TableHead>
              <TableBody>
                {chartList.map((chart, i) => (
                  <TableRow key={`entity-${i}`}>
                    <TableCell>{chart.title && translate(chart.title, chart.title)}</TableCell>
                    <TableCell>{chart.description && translate(chart.description, chart.description)}</TableCell>
                    <TableCell>{chart.chartView ? chart.chartView.name : ''}</TableCell>
                    <TableCell align="right">
                      <DropdownMenu icon={<MoreVertIcon fontSize="inherit" />}>
                        {isRootOrAdmin && (
                          <MenuItem component={Link} to={`${match.url}/${chart.id}/edit`}>
                            <Translate contentKey="entity.action.edit">Edit</Translate>
                          </MenuItem>
                        )}
                        {isRootOrAdmin && (
                          <MenuItem onClick={openDeleteDialog(chart)}>
                            <Translate contentKey="entity.action.delete">Delete</Translate>
                          </MenuItem>
                        )}
                        <MenuItem component={Link} to={`${match.url}/${chart.id}/visualize`}>
                          {isTypeMap(chart.chartView) ? (
                            <Translate contentKey="efraudyApp.chart.home.goToMap">Visualizza la mappa </Translate>
                          ) : (
                            <Translate contentKey="efraudyApp.chart.home.goToChart">Visualizza il grafico</Translate>
                          )}
                        </MenuItem>
                        {chart.addedBy && chart.addedBy.find(user => user.id === currentUser.id) ? (
                          <MenuItem onClick={() => props.removeUserFromChart(chart, props.currentUser.id)}>
                            {isTypeMap(chart.chartView) ? (
                              <Translate contentKey="efraudyApp.chart.home.removeMap">Rimuovi la mappa</Translate>
                            ) : (
                              <Translate contentKey="efraudyApp.chart.home.removeChart">Rimuovi il grafico</Translate>
                            )}
                          </MenuItem>
                        ) : (
                          <MenuItem onClick={() => props.updateEntity(chart, props.currentUser)}>
                            {isTypeMap(chart.chartView) ? (
                              <Translate contentKey="efraudyApp.chart.home.assignMap">Assegna la mappa</Translate>
                            ) : (
                              <Translate contentKey="efraudyApp.chart.home.assignChart">Assegna il grafico</Translate>
                            )}
                          </MenuItem>
                        )}
                      </DropdownMenu>
                    </TableCell>
                  </TableRow>
                ))}
              </TableBody>
            </Table>
          </TableContainer>
          <DialogBox
            title={translate('efraudyApp.chart.deleteChart')}
            open={isDeleteDialogOpen}
            handleClose={closeDeleteDialog}
            successHandler={deleteChart}
          >
            <Translate contentKey="efraudyApp.chart.delete.question" interpolate={{ id: entityToBeDeleted.title }}>
              Are you sure you want to delete Graph ?
            </Translate>
          </DialogBox>

          {totalItems > ITEMS_PER_PAGE ? (
            <Paginator
              activePage={pagination.activePage}
              onSelect={handlePagination}
              maxButtons={5}
              itemsPerPage={pagination.itemsPerPage}
              totalItems={totalItems}
            />
          ) : null}
        </div>
      ) : (
        !loading && (
          <AlertBox severity="warning">
            <Translate contentKey="efraudyApp.chart.home.notFound">No Rules found</Translate>
          </AlertBox>
        )
      )}
    </div>
  );
};

const mapStateToProps = ({ chart, authentication }: IRootState) => ({
  chartList: chart.entities,
  loading: chart.loading,
  totalItems: chart.totalItems,
  authorities: authentication.account.authorities,
  currentUser: authentication.account
});

const mapDispatchToProps = {
  getEntities,
  setButton,
  deleteEntity,
  resetSteps,
  updateEntity,
  removeUserFromChart
};

type StateProps = ReturnType<typeof mapStateToProps>;
type DispatchProps = typeof mapDispatchToProps;

export default connect(mapStateToProps, mapDispatchToProps)(Chart);
